require 'spec_helper'

module Wizypay
  RSpec.describe Merchant do
    let(:api_key) { 'key' }
    let(:api_secret) { 'secret' }
    let(:api_endpoint) { 'https://api.wizypay.com' }
    let(:all_merchants_request) { stub_request(:get, "#{api_endpoint}/merchants").
        with(headers: { 'Accept'        => 'application/json',
                        'Authorization' => /WIZYPAY #{api_key}:.*/ }) }
    let(:where_merchants_request) { stub_request(:get, "#{api_endpoint}/merchants?q=hp").
        with(headers: { 'Accept'        => 'application/json',
                        'Authorization' => /WIZYPAY #{api_key}:.*/ }) }
    before do
      Wizypay.setup(api_key, api_secret, api_endpoint)
    end

    context 'all' do
      it 'returns empty body' do
        request  = all_merchants_request.to_return(status: 200, body: '{"data": []}')
        response = Merchant.all
        expect(request).to have_been_made
        expect(response).to be_empty
      end

      it 'is a collection' do
        all_merchants_request.to_return(status: 200, body: '{"data": [], "meta": {"total": 0}}')
        response = Merchant.all
        expect(response).to be_a(Collection)
        expect(response.total).to eq 0
      end

      it 'is a collection of Merchants' do
        all_merchants_request.to_return(status: 200, body: '{"data": [{"id": 1, "name": "merchant 1"}]}')
        response = Merchant.all
        expect(response.first).to be_a(Merchant)
        expect(response.first.name).to eq 'merchant 1'
      end

      it 'has a primary ad' do
        all_merchants_request.to_return(status: 200, body: '{"data": [{"primary_ad": {"data": {"id": 1}}}]}')
        response = Merchant.all
        expect(response.first.primary_ad.id).to eq 1
      end

      it 'has categories' do
        all_merchants_request.to_return(status: 200, body: '{"data": [{"categories": {"data": [{"id": 1}]}}]}')
        response = Merchant.all
        expect(response.first.categories).to be_a(Collection)
        expect(response.first.categories.first).to be_a(MerchantCategory)
        expect(response.first.categories.first.id).to eq 1
      end
    end

    context 'where' do
      it 'returns empty body' do
        request  = where_merchants_request.to_return(status: 200, body: '{"data": []}')
        response = Merchant.where(q: 'hp')
        expect(request).to have_been_made
        expect(response).to be_empty
      end
    end

    context 'failure' do
      context '4XX' do
        it "throws #{RestClient::Exception}" do
          request = all_merchants_request.to_return(status: 401, body: { errors: { base: ['authentication_failed'] } }.to_json)
          expect do
            Merchant.all
          end.to raise_error(RestClient::Exception)
          expect(request).to have_been_made
        end
      end

      context 'timeout' do
        it "throws #{RestClient::Exception}" do
          request = all_merchants_request.to_timeout
          expect do
            Merchant.all
          end.to raise_error(RestClient::Exception)
          expect(request).to have_been_made
        end
      end
    end
  end
end
